package com.zipcodewilmington.phonebook;

import com.sun.org.apache.xpath.internal.operations.Bool;
import org.junit.Assert;
import org.junit.Test;

import java.lang.reflect.Array;
import java.util.ArrayList;

/**
 * Created by leon on 1/23/18.
 */
public class PhoneBookTest {

    @Test
    public void testAddLookup() {
        //Given
        PhoneBook myPhoneBook = new PhoneBook();
        String newName = "Hayden Stewart";
        String newNumber = "404-678-9989";
        ArrayList<String> expected = new ArrayList<>();
        expected.add(newNumber);

        //When
        myPhoneBook.add(newName, newNumber);
        ArrayList<String> actual = myPhoneBook.lookup(newName);

        //Then
        Assert.assertEquals(expected, actual);

    }

    @Test
    public void testAddtoName() {
        //Given
        PhoneBook myPhoneBook = new PhoneBook();
        String newName = "Hayden Stewart";
        String newNumber = "404-678-9989";
        String newNumber2 = "555-555-5589";
        ArrayList<String> expected = new ArrayList<>();
        expected.add(newNumber);
        expected.add(newNumber2);

        //When
        myPhoneBook.add(newName, newNumber);
        myPhoneBook.add(newName, newNumber2);
        ArrayList<String> actual = myPhoneBook.lookup(newName);

        //Then
        Assert.assertEquals(expected, actual);

    }

    @Test
    public void testRemoveLookup() {
        //Given
        PhoneBook myPhoneBook = new PhoneBook();
        String newName = "Hayden Stewart";
        String newNumber = "404-678-9989";
        String newNumber2 = "555-555-5589";
        ArrayList<String> expected = new ArrayList<>();
        expected.add(newNumber2);

        //When
        myPhoneBook.add(newName, newNumber);
        myPhoneBook.add(newName, newNumber2);
        myPhoneBook.remove(newName, newNumber);
        ArrayList<String> actual = myPhoneBook.lookup(newName);

        //Then
        Assert.assertEquals(expected, actual);

    }

    @Test
    public void testRemoveMultipleNumbers() {
        //Given
        PhoneBook myPhoneBook = new PhoneBook();
        String newName = "Hayden Stewart";
        String newNumber = "404-678-9989";
        String newNumber2 = "555-555-5589";
        ArrayList<String> expected = null;

        //When
        myPhoneBook.add(newName, newNumber);
        myPhoneBook.add(newName, newNumber2);
        myPhoneBook.removeRecord(newName);
        ArrayList<String> actual = myPhoneBook.lookup(newName);

        //Then
        Assert.assertEquals(expected, actual);

    }

    @Test
    public void testHasEntryName() {
        //Given
        PhoneBook myPhoneBook = new PhoneBook();
        String newName = "Hayden Stewart";
        String newNumber = "404-678-9989";
        Boolean expected = true;

        //When
        myPhoneBook.add(newName, newNumber);
        Boolean actual = myPhoneBook.hasEntryName(newName);

        //Then
        Assert.assertEquals(expected, actual);

    }

    @Test
    public void testArrayListForNumbers() {
        //Given
        PhoneBook book = new PhoneBook();
        book.add("Bob Smith", "999-234-3432");
        book.add("Kate Flynn", "678-234-3432");
        ArrayList<ArrayList<String>> expected = new ArrayList<>();
        ArrayList<String> person1 = new ArrayList<>();
        ArrayList<String> person2 = new ArrayList<>();
        person1.add("999-234-3432");
        person2.add("678-234-3432");
        expected.add(person1);
        expected.add(person2);

        //When
        ArrayList<ArrayList<String>> actual = book.getArrayListForNumbers();

        //Then
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void testArrayListForNames() {
        //Given
        PhoneBook book = new PhoneBook();
        book.add("Bob Smith", "999-234-3432");
        book.add("Kate Flynn", "678-234-3432");
        book.add("Liz Green", "100-234-3432");
        book.add("Abby Schwartz", "088-234-3432");
        String expected = "[Abby Schwartz, Bob Smith, Kate Flynn, Liz Green]";

        //When
        String actual = book.getArrayListForNames().toString();

        //Then
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void testListNamesNumbers() {
        //Given
        PhoneBook book = new PhoneBook();
        book.add("Bob Smith", "999-234-3432");
        book.add("Kate Flynn", "678-234-3432");
        book.add("Liz Green", "100-234-3432");
        book.add("Abby Schwartz", "088-234-3432");
        String expected = String.format("%20s %14s", "Abby Schwartz", "[088-234-3432]") + "\n" +
                String.format("%20s %14s", "Bob Smith", "[999-234-3432]") + "\n" +
                String.format("%20s %14s", "Kate Flynn", "[678-234-3432]") + "\n" +
                String.format("%20s %14s", "Liz Green", "[100-234-3432]") + "\n";

        //When
        String actual = book.listNamesAndNumbers();

        //Then
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void testReverseLookup() {
        //Given
        PhoneBook book = new PhoneBook();
        book.add("Bob Smith", "999-234-3432");
        book.add("Kate Flynn", "678-234-3432");
        book.add("Liz Green", "100-234-3432");
        book.add("Abby Schwartz", "088-234-3432");
        String expected = "Liz Green";

        //When
        String actual = book.reverseLookup("100-234-3432");

        //Then
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void testReverseLookupMultipleNumbers() {
        //Given
        PhoneBook book = new PhoneBook();
        book.add("Bob Smith", "999-234-3432");
        book.add("Liz Green", "678-234-3432");
        book.add("Liz Green", "100-234-3432");
        book.add("Bob Smith", "088-234-3432");
        String expected = "Liz Green";

        //When
        String actual = book.reverseLookup("100-234-3432");

        //Then
        Assert.assertEquals(expected, actual);
    }
}
